@@ -16,7 +16,7 @@
 #include "header.h"
 #include "types.h"
 
-#define PCI_LIB_VERSION 0x030604
+#define PCI_LIB_VERSION 0x030800
 
 #ifndef PCI_ABI
 #define PCI_ABI
@@ -43,6 +43,7 @@ enum pci_access_type {
   PCI_ACCESS_DARWIN,			/* Darwin */
   PCI_ACCESS_SYLIXOS_DEVICE,		/* SylixOS pci */
   PCI_ACCESS_HURD,			/* GNU/Hurd */
+  PCI_ACCESS_WIN32_CFGMGR32,		/* Win32 cfgmgr32.dll */
   PCI_ACCESS_MAX
 };
 
@@ -62,7 +63,7 @@ struct pci_access {
   int debugging;			/* Turn on debugging messages */
 
   /* Functions you can override: */
-  void (*error)(char *msg, ...) PCI_PRINTF(1,2);	/* Write error message and quit */
+  void (*error)(char *msg, ...) PCI_PRINTF(1,2) PCI_NONRET;	/* Write error message and quit */
   void (*warning)(char *msg, ...) PCI_PRINTF(1,2);	/* Write a warning message */
   void (*debug)(char *msg, ...) PCI_PRINTF(1,2);	/* Write a debugging message */
 
@@ -126,7 +127,7 @@ struct pci_dev {
   u8 bus, dev, func;			/* Bus inside domain, device and function */
 
   /* These fields are set by pci_fill_info() */
-  int known_fields;			/* Set of info fields already known */
+  unsigned int known_fields;		/* Set of info fields already known (see pci_fill_info()) */
   u16 vendor_id, device_id;		/* Identity of the device */
   u16 device_class;			/* PCI device class */
   int irq;				/* IRQ number */
@@ -142,6 +143,13 @@ struct pci_dev {
   pciaddr_t flags[6];			/* PCI_IORESOURCE_* flags for regions */
   pciaddr_t rom_flags;			/* PCI_IORESOURCE_* flags for expansion ROM */
   int domain;				/* PCI domain (host bridge) */
+  pciaddr_t bridge_base_addr[4];	/* Bridge base addresses (without flags) */
+  pciaddr_t bridge_size[4];		/* Bridge sizes */
+  pciaddr_t bridge_flags[4];		/* PCI_IORESOURCE_* flags for bridge addresses */
+  u8 prog_if, rev_id;			/* Programming interface for device_class and revision id */
+  u16 subsys_vendor_id, subsys_id;	/* Subsystem vendor id and subsystem id */
+  struct pci_dev *parent;		/* Parent device, does not have to be always accessible */
+  int no_config_access;			/* No access to config space for this device */
 
   /* Fields used internally */
   struct pci_access *access;
@@ -149,7 +157,7 @@ struct pci_dev {
   u8 *cache;				/* Cached config registers */
   int cache_len;
   int hdrtype;				/* Cached low 7 bits of header type, -1 if unknown */
-  void *aux;				/* Auxiliary data */
+  void *aux;				/* Auxiliary data for use by the back-end */
   struct pci_property *properties;	/* A linked list of extra properties */
   struct pci_cap *last_cap;		/* Last capability in the list */
 };
@@ -175,6 +183,16 @@ int pci_write_block(struct pci_dev *, int pos, u8 *buf, int len) PCI_ABI;
  *
  * Some properties are stored directly in the pci_dev structure.
  * The remaining ones can be accessed through pci_get_string_property().
+ *
+ * pci_fill_info() returns the current value of pci_dev->known_fields.
+ * This is a bit mask of all fields, which were already obtained during
+ * the lifetime of the device. This includes fields which are not supported
+ * by the particular device -- in that case, the field is left at its default
+ * value, which is 0 for integer fields and NULL for pointers. On the other
+ * hand, we never consider known fields unsupported by the current back-end;
+ * such fields always contain the default value.
+ *
+ * XXX: flags and the result should be unsigned, but we do not want to break the ABI.
  */
 
 int pci_fill_info(struct pci_dev *, int flags) PCI_ABI;
@@ -194,7 +212,13 @@ char *pci_get_string_property(struct pci_dev *d, u32 prop) PCI_ABI;
 #define PCI_FILL_NUMA_NODE	0x0800
 #define PCI_FILL_IO_FLAGS	0x1000
 #define PCI_FILL_DT_NODE	0x2000		/* Device tree node */
+#define PCI_FILL_IOMMU_GROUP	0x4000
+#define PCI_FILL_BRIDGE_BASES	0x8000
 #define PCI_FILL_RESCAN		0x00010000
+#define PCI_FILL_CLASS_EXT	0x00020000      /* prog_if and rev_id */
+#define PCI_FILL_SUBSYS		0x00040000      /* subsys_vendor_id and subsys_id */
+#define PCI_FILL_PARENT		0x00080000
+#define PCI_FILL_DRIVER		0x00100000      /* OS driver currently in use (string property) */
 
 void pci_setup_cache(struct pci_dev *, u8 *cache, int len) PCI_ABI;
 
@@ -222,8 +246,11 @@ struct pci_cap *pci_find_cap_nr(struct pci_dev *, unsigned int id, unsigned int
 
 struct pci_filter {
   int domain, bus, slot, func;			/* -1 = ANY */
-  int vendor, device, device_class;
-  int rfu[3];
+  int vendor, device;
+  int device_class;
+  unsigned int device_class_mask;		/* Which bits of the device_class are compared, default=all */
+  int prog_if;
+  int rfu[1];
 };
 
 void pci_filter_init(struct pci_access *, struct pci_filter *) PCI_ABI;
